@@ -26,8 +26,10 @@
 #include "autotestplugin.h"
 #include "autotestconstants.h"
 #include "autotesticons.h"
+#include "projectsettingswidget.h"
 #include "testcodeparser.h"
 #include "testframeworkmanager.h"
+#include "testprojectsettings.h"
 #include "testrunner.h"
 #include "testsettings.h"
 #include "testsettingspage.h"
@@ -40,6 +42,8 @@
 #include "qtest/qttestframework.h"
 #include "quick/quicktestframework.h"
 #include "gtest/gtestframework.h"
+#include "boost/boosttestframework.h"
+#include "catch/catchframework.h"
 
 #include <coreplugin/icore.h>
 #include <coreplugin/icontext.h>
@@ -53,54 +57,143 @@
 #include <projectexplorer/buildmanager.h>
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/projectexplorericons.h>
+#include <projectexplorer/projectpanelfactory.h>
+#include <projectexplorer/runconfiguration.h>
 #include <projectexplorer/session.h>
 #include <projectexplorer/target.h>
 #include <texteditor/texteditor.h>
+#include <texteditor/textdocument.h>
 #include <utils/textutils.h>
 #include <utils/utilsicons.h>
 
 #include <QAction>
 #include <QList>
+#include <QMap>
 #include <QMessageBox>
 #include <QMainWindow>
 #include <QMenu>
 #include <QTextCursor>
-#include <QtPlugin>
 
 #ifdef WITH_TESTS
 #include "autotestunittests.h"
 #endif
 
-using namespace Autotest::Internal;
 using namespace Core;
 
-static AutotestPlugin *s_instance = nullptr;
+namespace Autotest {
+namespace Internal {
+
+class AutotestPluginPrivate : public QObject
+{
+    Q_OBJECT
+public:
+    AutotestPluginPrivate();
+    ~AutotestPluginPrivate() override;
+
+    TestNavigationWidgetFactory *m_navigationWidgetFactory = nullptr;
+    TestResultsPane *m_resultsPane = nullptr;
+    QMap<QString, ChoicePair> m_runconfigCache;
+
+    void initializeMenuEntries();
+    void onRunAllTriggered();
+    void onRunSelectedTriggered();
+    void onRunFileTriggered();
+    void onRunUnderCursorTriggered(TestRunMode mode);
+
+    TestSettings m_settings;
+    TestSettingsPage m_testSettingPage{&m_settings};
+
+    TestCodeParser m_testCodeParser;
+    TestTreeModel m_testTreeModel{&m_testCodeParser};
+    TestRunner m_testRunner;
+    TestFrameworkManager m_frameworkManager;
+};
+
+static AutotestPluginPrivate *dd = nullptr;
+static QHash<ProjectExplorer::Project *, TestProjectSettings *> s_projectSettings;
 
 AutotestPlugin::AutotestPlugin()
-    : m_settings(new TestSettings)
 {
     // needed to be used in QueuedConnection connects
     qRegisterMetaType<TestResult>();
     qRegisterMetaType<TestTreeItem *>();
     qRegisterMetaType<TestCodeLocationAndType>();
-
-    s_instance = this;
+    // warm up meta type system to be able to read Qt::CheckState with persistent settings
+    qRegisterMetaType<Qt::CheckState>();
 }
 
 AutotestPlugin::~AutotestPlugin()
 {
+    delete dd;
+    dd = nullptr;
+}
+
+AutotestPluginPrivate::AutotestPluginPrivate()
+{
+    dd = this; // Needed as the code below access it via the static plugin interface
+    initializeMenuEntries();
+    m_frameworkManager.registerTestFramework(new QtTestFramework);
+    m_frameworkManager.registerTestFramework(new QuickTestFramework);
+    m_frameworkManager.registerTestFramework(new GTestFramework);
+    m_frameworkManager.registerTestFramework(new BoostTestFramework);
+    m_frameworkManager.registerTestFramework(new CatchFramework);
+
+    m_frameworkManager.synchronizeSettings(ICore::settings());
+    m_navigationWidgetFactory = new TestNavigationWidgetFactory;
+    m_resultsPane = TestResultsPane::instance();
+
+    auto panelFactory = new ProjectExplorer::ProjectPanelFactory();
+    panelFactory->setPriority(666);
+//    panelFactory->setIcon();  // TODO ?
+    panelFactory->setDisplayName(tr("Testing"));
+    panelFactory->setCreateWidgetFunction([](ProjectExplorer::Project *project) {
+        return new ProjectTestSettingsWidget(project);
+    });
+    ProjectExplorer::ProjectPanelFactory::registerFactory(panelFactory);
+
+    m_frameworkManager.activateFrameworksFromSettings(&m_settings);
+    m_testTreeModel.synchronizeTestFrameworks();
+
+    auto sessionManager = ProjectExplorer::SessionManager::instance();
+    connect(sessionManager, &ProjectExplorer::SessionManager::startupProjectChanged,
+            this, [this] { m_runconfigCache.clear(); });
+
+    connect(sessionManager, &ProjectExplorer::SessionManager::aboutToRemoveProject,
+            this, [] (ProjectExplorer::Project *project) {
+        auto it = s_projectSettings.find(project);
+        if (it != s_projectSettings.end()) {
+            delete it.value();
+            s_projectSettings.erase(it);
+        }
+    });
+}
+
+AutotestPluginPrivate::~AutotestPluginPrivate()
+{
+    if (!s_projectSettings.isEmpty()) {
+        qDeleteAll(s_projectSettings.values());
+        s_projectSettings.clear();
+    }
+
     delete m_navigationWidgetFactory;
     delete m_resultsPane;
-    delete m_testSettingPage;
-    delete m_frameworkManager;
 }
 
-QSharedPointer<TestSettings> AutotestPlugin::settings()
+TestSettings *AutotestPlugin::settings()
 {
-    return s_instance->m_settings;
+    return &dd->m_settings;
 }
 
-void AutotestPlugin::initializeMenuEntries()
+TestProjectSettings *AutotestPlugin::projectSettings(ProjectExplorer::Project *project)
+{
+    auto &settings = s_projectSettings[project];
+    if (!settings)
+        settings = new TestProjectSettings(project);
+
+    return settings;
+}
+
+void AutotestPluginPrivate::initializeMenuEntries()
 {
     ActionContainer *menu = ActionManager::createMenu(Constants::MENU_ID);
     menu->menu()->setTitle(tr("&Tests"));
@@ -112,7 +205,7 @@ void AutotestPlugin::initializeMenuEntries()
     Command *command = ActionManager::registerAction(action, Constants::ACTION_RUN_ALL_ID);
     command->setDefaultKeySequence(
         QKeySequence(useMacShortcuts ? tr("Ctrl+Meta+T, Ctrl+Meta+A") : tr("Alt+Shift+T,Alt+A")));
-    connect(action, &QAction::triggered, this, &AutotestPlugin::onRunAllTriggered);
+    connect(action, &QAction::triggered, this, &AutotestPluginPrivate::onRunAllTriggered);
     action->setEnabled(false);
     menu->addAction(command);
 
@@ -125,7 +218,7 @@ void AutotestPlugin::initializeMenuEntries()
     command = ActionManager::registerAction(action, Constants::ACTION_RUN_SELECTED_ID);
     command->setDefaultKeySequence(
         QKeySequence(useMacShortcuts ? tr("Ctrl+Meta+T, Ctrl+Meta+R") : tr("Alt+Shift+T,Alt+R")));
-    connect(action, &QAction::triggered, this, &AutotestPlugin::onRunSelectedTriggered);
+    connect(action, &QAction::triggered, this, &AutotestPluginPrivate::onRunSelectedTriggered);
     action->setEnabled(false);
     menu->addAction(command);
 
@@ -138,7 +231,7 @@ void AutotestPlugin::initializeMenuEntries()
     command = ActionManager::registerAction(action, Constants::ACTION_RUN_FILE_ID);
     command->setDefaultKeySequence(
         QKeySequence(useMacShortcuts ? tr("Ctrl+Meta+T, Ctrl+Meta+F") : tr("Alt+Shift+T,Alt+F")));
-    connect(action, &QAction::triggered, this, &AutotestPlugin::onRunFileTriggered);
+    connect(action, &QAction::triggered, this, &AutotestPluginPrivate::onRunFileTriggered);
     action->setEnabled(false);
     menu->addAction(command);
 
@@ -146,9 +239,8 @@ void AutotestPlugin::initializeMenuEntries()
     command = ActionManager::registerAction(action, Constants::ACTION_SCAN_ID);
     command->setDefaultKeySequence(
         QKeySequence(useMacShortcuts ? tr("Ctrl+Meta+T, Ctrl+Meta+S") : tr("Alt+Shift+T,Alt+S")));
-    connect(action, &QAction::triggered, this, []() {
-        TestTreeModel::instance()->parser()->updateTestTree();
-    });
+
+    connect(action, &QAction::triggered, this, [] { dd->m_testCodeParser.updateTestTree(); });
     menu->addAction(command);
 
     ActionContainer *toolsMenu = ActionManager::actionContainer(Core::Constants::M_TOOLS);
@@ -158,9 +250,9 @@ void AutotestPlugin::initializeMenuEntries()
             this, &AutotestPlugin::updateMenuItemsEnabledState);
     connect(BuildManager::instance(), &BuildManager::buildQueueFinished,
             this, &AutotestPlugin::updateMenuItemsEnabledState);
-    connect(ProjectExplorerPlugin::instance(), &ProjectExplorerPlugin::updateRunActions,
+    connect(ProjectExplorerPlugin::instance(), &ProjectExplorerPlugin::runActionsUpdated,
             this, &AutotestPlugin::updateMenuItemsEnabledState);
-    connect(TestTreeModel::instance(), &TestTreeModel::testTreeModelChanged,
+    connect(&dd->m_testTreeModel, &TestTreeModel::testTreeModelChanged,
             this, &AutotestPlugin::updateMenuItemsEnabledState);
 }
 
@@ -169,20 +261,7 @@ bool AutotestPlugin::initialize(const QStringList &arguments, QString *errorStri
     Q_UNUSED(arguments)
     Q_UNUSED(errorString)
 
-    m_frameworkManager = TestFrameworkManager::instance();
-    initializeMenuEntries();
-    m_frameworkManager->registerTestFramework(new QtTestFramework);
-    m_frameworkManager->registerTestFramework(new QuickTestFramework);
-    m_frameworkManager->registerTestFramework(new GTestFramework);
-
-    m_frameworkManager->synchronizeSettings(ICore::settings());
-    m_testSettingPage = new TestSettingsPage(m_settings);
-    m_navigationWidgetFactory = new TestNavigationWidgetFactory;
-    m_resultsPane = TestResultsPane::instance();
-
-    m_frameworkManager->activateFrameworksFromSettings(m_settings);
-    TestTreeModel::instance()->syncTestFrameworks();
-
+    dd = new AutotestPluginPrivate;
     return true;
 }
 
@@ -194,104 +273,112 @@ void AutotestPlugin::extensionsInitialized()
 
     QAction *action = new QAction(tr("&Run Test Under Cursor"), this);
     action->setEnabled(false);
-    action->setIcon(Utils::Icons::RUN_SMALL_TOOLBAR.icon());
+    action->setIcon(Utils::Icons::RUN_SMALL.icon());
 
     Command *command = ActionManager::registerAction(action, Constants::ACTION_RUN_UCURSOR);
-    connect(action, &QAction::triggered, std::bind(&AutotestPlugin::onRunUnderCursorTriggered, this,
-                                                   TestRunMode::Run));
+    connect(action, &QAction::triggered,
+            std::bind(&AutotestPluginPrivate::onRunUnderCursorTriggered, dd, TestRunMode::Run));
     contextMenu->addSeparator();
     contextMenu->addAction(command);
 
-    action = new QAction(tr("&Debug Test Under Cursor"), this);;
+    action = new QAction(tr("&Debug Test Under Cursor"), this);
     action->setEnabled(false);
     action->setIcon(ProjectExplorer::Icons::DEBUG_START_SMALL.icon());
 
-
     command = ActionManager::registerAction(action, Constants::ACTION_RUN_DBG_UCURSOR);
-    connect(action, &QAction::triggered, std::bind(&AutotestPlugin::onRunUnderCursorTriggered, this,
-                                                   TestRunMode::Debug));
+    connect(action, &QAction::triggered,
+            std::bind(&AutotestPluginPrivate::onRunUnderCursorTriggered, dd, TestRunMode::Debug));
     contextMenu->addAction(command);
     contextMenu->addSeparator();
 }
 
 ExtensionSystem::IPlugin::ShutdownFlag AutotestPlugin::aboutToShutdown()
 {
-    TestTreeModel::instance()->parser()->aboutToShutdown();
+    dd->m_testCodeParser.aboutToShutdown();
+    dd->m_testTreeModel.disconnect();
     return SynchronousShutdown;
 }
 
-void AutotestPlugin::onRunAllTriggered()
+void AutotestPluginPrivate::onRunAllTriggered()
 {
-    TestRunner *runner = TestRunner::instance();
-    TestTreeModel *model = TestTreeModel::instance();
-    runner->setSelectedTests(model->getAllTestCases());
-    runner->prepareToRunTests(TestRunMode::Run);
+    m_testRunner.setSelectedTests(m_testTreeModel.getAllTestCases());
+    m_testRunner.prepareToRunTests(TestRunMode::Run);
 }
 
-void AutotestPlugin::onRunSelectedTriggered()
+void AutotestPluginPrivate::onRunSelectedTriggered()
 {
-    TestRunner *runner = TestRunner::instance();
-    TestTreeModel *model = TestTreeModel::instance();
-    runner->setSelectedTests(model->getSelectedTests());
-    runner->prepareToRunTests(TestRunMode::Run);
+    m_testRunner.setSelectedTests(m_testTreeModel.getSelectedTests());
+    m_testRunner.prepareToRunTests(TestRunMode::Run);
 }
 
-void AutotestPlugin::onRunFileTriggered()
+void AutotestPluginPrivate::onRunFileTriggered()
 {
     const IDocument *document = EditorManager::currentDocument();
     if (!document)
         return;
 
-    const Utils::FileName &fileName = document->filePath();
+    const Utils::FilePath &fileName = document->filePath();
     if (fileName.isEmpty())
         return;
 
-    TestTreeModel *model = TestTreeModel::instance();
-    const QList<TestConfiguration *> tests = model->getTestsForFile(fileName);
+    const QList<TestConfiguration *> tests = m_testTreeModel.getTestsForFile(fileName);
     if (tests.isEmpty())
         return;
 
-    TestRunner *runner = TestRunner::instance();
-    runner->setSelectedTests(tests);
-    runner->prepareToRunTests(TestRunMode::Run);
+    m_testRunner.setSelectedTests(tests);
+    m_testRunner.prepareToRunTests(TestRunMode::Run);
 }
 
-void AutotestPlugin::onRunUnderCursorTriggered(TestRunMode mode)
+static QList<TestConfiguration *> testItemsToTestConfigurations(const QList<TestTreeItem *> &items,
+                                                                TestRunMode mode)
 {
-    QTextCursor cursor = Utils::Text::wordStartCursor(
-                             TextEditor::BaseTextEditor::currentTextEditor()->editorWidget()->textCursor());
-    cursor.movePosition(QTextCursor::EndOfWord, QTextCursor::KeepAnchor);
+    QList<TestConfiguration *> configs;
+    for (const TestTreeItem * item : items) {
+        if (TestConfiguration *currentConfig = item->asConfiguration(mode))
+            configs << currentConfig;
+    }
+    return configs;
+}
+
+void AutotestPluginPrivate::onRunUnderCursorTriggered(TestRunMode mode)
+{
+    TextEditor::BaseTextEditor *currentEditor = TextEditor::BaseTextEditor::currentTextEditor();
+    QTextCursor cursor = currentEditor->editorWidget()->textCursor();
+    cursor.select(QTextCursor::WordUnderCursor);
     const QString text = cursor.selectedText();
     if (text.isEmpty())
         return; // Do not trigger when no name under cursor
 
-    const QList<TestTreeItem *> testsItems = TestTreeModel::instance()->testItemsByName(text);
+    const QList<TestTreeItem *> testsItems = m_testTreeModel.testItemsByName(text);
     if (testsItems.isEmpty())
         return; // Wrong location triggered
 
-    QList<TestConfiguration *> testsToRun;
-    for (const TestTreeItem * item : testsItems){
-        if (TestConfiguration *cfg = item->asConfiguration(mode))
-            testsToRun << cfg;
-    }
+    // check whether we have been triggered on a test function definition
+    const int line = currentEditor->currentLine();
+    const QString &filePath = currentEditor->textDocument()->filePath().toString();
+    const QList<TestTreeItem *> filteredItems = Utils::filtered(testsItems, [&](TestTreeItem *it){
+        return it->line() == line && it->filePath() == filePath;
+    });
+
+    const QList<TestConfiguration *> testsToRun = testItemsToTestConfigurations(
+                filteredItems.size() == 1 ? filteredItems : testsItems, mode);
 
     if (testsToRun.isEmpty()) {
         MessageManager::write(tr("Selected test was not found (%1).").arg(text), MessageManager::Flash);
         return;
     }
 
-    auto runner = TestRunner::instance();
-    runner->setSelectedTests(testsToRun);
-    runner->prepareToRunTests(mode);
+    m_testRunner.setSelectedTests(testsToRun);
+    m_testRunner.prepareToRunTests(mode);
 }
 
 void AutotestPlugin::updateMenuItemsEnabledState()
 {
     const ProjectExplorer::Project *project = ProjectExplorer::SessionManager::startupProject();
     const ProjectExplorer::Target *target = project ? project->activeTarget() : nullptr;
-    const bool canScan = !TestRunner::instance()->isTestRunning()
-            && TestTreeModel::instance()->parser()->state() == TestCodeParser::Idle;
-    const bool hasTests = TestTreeModel::instance()->hasTests();
+    const bool canScan = !dd->m_testRunner.isTestRunning()
+            && dd->m_testCodeParser.state() == TestCodeParser::Idle;
+    const bool hasTests = dd->m_testTreeModel.hasTests();
     // avoid expensive call to PE::canRunStartupProject() - limit to minimum necessary checks
     const bool canRun = hasTests && canScan
             && project && !project->needsConfiguration()
@@ -311,11 +398,44 @@ void AutotestPlugin::updateMenuItemsEnabledState()
     ActionManager::command(Constants::ACTION_RUN_DBG_UCURSOR)->action()->setEnabled(canRun);
 }
 
-QList<QObject *> AutotestPlugin::createTestObjects() const
+void AutotestPlugin::cacheRunConfigChoice(const QString &buildTargetKey, const ChoicePair &choice)
 {
-    QList<QObject *> tests;
+    if (dd)
+        dd->m_runconfigCache.insert(buildTargetKey, choice);
+}
+
+ChoicePair AutotestPlugin::cachedChoiceFor(const QString &buildTargetKey)
+{
+    return dd ? dd->m_runconfigCache.value(buildTargetKey) : ChoicePair();
+}
+
+void AutotestPlugin::clearChoiceCache()
+{
+    if (dd)
+        dd->m_runconfigCache.clear();
+}
+
+void AutotestPlugin::popupResultsPane()
+{
+    if (dd)
+        dd->m_resultsPane->popup(Core::IOutputPane::NoModeSwitch);
+}
+
+QVector<QObject *> AutotestPlugin::createTestObjects() const
+{
+    QVector<QObject *> tests;
 #ifdef WITH_TESTS
-    tests << new AutoTestUnitTests(TestTreeModel::instance());
+    tests << new AutoTestUnitTests(&dd->m_testTreeModel);
 #endif
     return tests;
 }
+
+bool ChoicePair::matches(const ProjectExplorer::RunConfiguration *rc) const
+{
+    return rc && rc->displayName() == displayName && rc->runnable().executable.toString() == executable;
+}
+
+} // Internal
+} // Autotest
+
+#include "autotestplugin.moc"
