@@ -87,7 +87,7 @@ McuPackage::McuPackage(const QString &label, const QString &defaultPath,
 
 QString McuPackage::path() const
 {
-    return QFileInfo(m_fileChooser->path() + m_relativePathModifier).absoluteFilePath();
+    return QFileInfo(m_fileChooser->filePath().toString() + m_relativePathModifier).absoluteFilePath();
 }
 
 QString McuPackage::label() const
@@ -125,7 +125,7 @@ QWidget *McuPackage::widget()
 
     if (!m_downloadUrl.isEmpty()) {
         auto downLoadButton = new QToolButton;
-        downLoadButton->setIcon(Utils::Icons::DOWNLOAD.icon());
+        downLoadButton->setIcon(Utils::Icons::ONLINE.icon());
         downLoadButton->setToolTip(tr("Download from \"%1\"").arg(m_downloadUrl));
         QObject::connect(downLoadButton, &QToolButton::pressed, [this]{
             QDesktopServices::openUrl(m_downloadUrl);
@@ -199,7 +199,7 @@ void McuPackage::updateStatus()
     m_path = m_fileChooser->rawPath();
     const bool validPath = m_fileChooser->isValid();
     const Utils::FilePath detectionPath = Utils::FilePath::fromString(
-                m_fileChooser->path() + "/" + m_detectionPath);
+                m_fileChooser->filePath().toString() + "/" + m_detectionPath);
     const QString displayDetectionPath = Utils::FilePath::fromString(m_detectionPath).toUserOutput();
     const bool validPackage = m_detectionPath.isEmpty() || detectionPath.exists();
 
@@ -240,7 +240,7 @@ McuToolChainPackage::Type McuToolChainPackage::type() const
     return m_type;
 }
 
-static ProjectExplorer::ToolChain *desktopToolChain(Core::Id language)
+static ProjectExplorer::ToolChain *desktopToolChain(Utils::Id language)
 {
     using namespace ProjectExplorer;
 
@@ -256,7 +256,7 @@ static ProjectExplorer::ToolChain *desktopToolChain(Core::Id language)
     return toolChain;
 }
 
-static ProjectExplorer::ToolChain* armGccToolChain(const Utils::FilePath &path, Core::Id language)
+static ProjectExplorer::ToolChain* armGccToolChain(const Utils::FilePath &path, Utils::Id language)
 {
     using namespace ProjectExplorer;
 
@@ -282,7 +282,7 @@ static ProjectExplorer::ToolChain* armGccToolChain(const Utils::FilePath &path,
     return toolChain;
 }
 
-ProjectExplorer::ToolChain *McuToolChainPackage::toolChain(Core::Id language) const
+ProjectExplorer::ToolChain *McuToolChainPackage::toolChain(Utils::Id language) const
 {
     ProjectExplorer::ToolChain *tc = nullptr;
     if (m_type == TypeDesktop) {
@@ -512,7 +512,7 @@ static void setKitProperties(const QString &kitName, ProjectExplorer::Kit *k,
     k->makeSticky();
     if (mcuTarget->toolChainPackage()->type() == McuToolChainPackage::TypeDesktop)
         k->setDeviceTypeForIcon(DEVICE_TYPE);
-    QSet<Core::Id> irrelevant = {
+    QSet<Utils::Id> irrelevant = {
         SysRootKitAspect::id(),
         QtSupport::QtKitAspect::id()
     };
@@ -607,7 +607,9 @@ static void setKitCMakeOptions(ProjectExplorer::Kit *k, const McuTarget* mcuTarg
                                   (qulDir + "/lib/cmake/Qul/QulGenerators.cmake").toUtf8()));
     config.append(CMakeConfigItem("QUL_PLATFORM",
                                   mcuTarget->qulPlatform().toUtf8()));
-    if (mcuTarget->os() == McuTarget::OS::FreeRTOS)
+
+    if (mcuTarget->qulVersion() <= QVersionNumber{1,3} // OS variable was removed in Qul 1.4
+        && mcuTarget->os() == McuTarget::OS::FreeRTOS)
         config.append(CMakeConfigItem("OS", "FreeRTOS"));
     if (mcuTarget->colorDepth() >= 0)
         config.append(CMakeConfigItem("QUL_COLOR_DEPTH",
@@ -627,8 +629,12 @@ static void setKitQtVersionOptions(ProjectExplorer::Kit *k)
 
 QString McuSupportOptions::kitName(const McuTarget *mcuTarget)
 {
-    const QString os = QLatin1String(mcuTarget->os()
-                                     == McuTarget::OS::FreeRTOS ? " FreeRTOS" : "");
+    QString os;
+    if (mcuTarget->qulVersion() <= QVersionNumber{1,3} && mcuTarget->os() == McuTarget::OS::FreeRTOS) {
+        // Starting from Qul 1.4 each OS is a separate platform
+        os = QLatin1String(" FreeRTOS");
+    }
+
     const QString colorDepth = mcuTarget->colorDepth() > 0
             ? QString::fromLatin1(" %1bpp").arg(mcuTarget->colorDepth())
             : "";
@@ -637,8 +643,12 @@ QString McuSupportOptions::kitName(const McuTarget *mcuTarget)
             mcuTarget->toolChainPackage()->type() == McuToolChainPackage::TypeDesktop
             ? "Desktop"
             : mcuTarget->qulPlatform();
-    return QString::fromLatin1("Qt for MCUs %1 - %2%3%4")
-            .arg(mcuTarget->qulVersion().toString(), targetName, os, colorDepth);
+    return QString::fromLatin1("Qt for MCUs %1.%2 - %3%4%5")
+            .arg(QString::number(mcuTarget->qulVersion().majorVersion()),
+                 QString::number(mcuTarget->qulVersion().minorVersion()),
+                 targetName,
+                 os,
+                 colorDepth);
 }
 
 QList<ProjectExplorer::Kit *> McuSupportOptions::existingKits(const McuTarget *mcuTarget)
