@@ -42,7 +42,7 @@
 #include <QVBoxLayout>
 #include <QLineEdit>
 #include <QLabel>
-#include <QRegExpValidator>
+#include <QRegularExpressionValidator>
 #include <QComboBox>
 #include <QTextEdit>
 #include <QSpacerItem>
@@ -69,17 +69,17 @@ namespace Internal {
     \sa ProjectExplorer::CustomWizard
 */
 
-CustomWizardFieldPage::LineEditData::LineEditData(QLineEdit* le, const QString &defText, const QString &pText) :
+CustomWizardFieldPage::LineEditData::LineEditData(QLineEdit *le, const QString &defText, const QString &pText) :
     lineEdit(le), defaultText(defText), placeholderText(pText)
 {
 }
 
-CustomWizardFieldPage::TextEditData::TextEditData(QTextEdit* le, const QString &defText) :
+CustomWizardFieldPage::TextEditData::TextEditData(QTextEdit *le, const QString &defText) :
     textEdit(le), defaultText(defText)
 {
 }
 
-CustomWizardFieldPage::PathChooserData::PathChooserData(PathChooser* pe, const QString &defText) :
+CustomWizardFieldPage::PathChooserData::PathChooserData(PathChooser *pe, const QString &defText) :
     pathChooser(pe), defaultText(defText)
 {
 }
@@ -93,7 +93,7 @@ CustomWizardFieldPage::CustomWizardFieldPage(const QSharedPointer<CustomWizardCo
     m_formLayout(new QFormLayout),
     m_errorLabel(new QLabel)
 {
-    QVBoxLayout *vLayout = new QVBoxLayout;
+    auto vLayout = new QVBoxLayout;
     m_formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
     if (debug)
         qDebug() << Q_FUNC_INFO << parameters->fields.size();
@@ -109,10 +109,6 @@ CustomWizardFieldPage::CustomWizardFieldPage(const QSharedPointer<CustomWizardCo
         setTitle(parameters->fieldPageTitle);
 }
 
-CustomWizardFieldPage::~CustomWizardFieldPage()
-{
-}
-
 void CustomWizardFieldPage::addRow(const QString &name, QWidget *w)
 {
     m_formLayout->addRow(name, w);
@@ -144,7 +140,7 @@ void CustomWizardFieldPage::addField(const CustomWizardField &field)\
     bool spansRow = false;
     // Check known classes: QComboBox
     const QString className = field.controlAttributes.value(QLatin1String("class"));
-    QWidget *fieldWidget = 0;
+    QWidget *fieldWidget = nullptr;
     if (className == QLatin1String("QComboBox")) {
         fieldWidget = registerComboBox(fieldName, field);
     } else if (className == QLatin1String("QTextEdit")) {
@@ -167,7 +163,7 @@ void CustomWizardFieldPage::addField(const CustomWizardField &field)\
 static void comboChoices(const CustomWizardField::ControlAttributeMap &controlAttributes,
                   QStringList *values, QStringList *displayTexts)
 {
-    typedef CustomWizardField::ControlAttributeMap::ConstIterator AttribMapConstIt;
+    using AttribMapConstIt = CustomWizardField::ControlAttributeMap::ConstIterator;
 
     values->clear();
     displayTexts->clear();
@@ -196,7 +192,7 @@ static void comboChoices(const CustomWizardField::ControlAttributeMap &controlAt
 QWidget *CustomWizardFieldPage::registerComboBox(const QString &fieldName,
                                                  const CustomWizardField &field)
 {
-    TextFieldComboBox *combo = new TextFieldComboBox;
+    auto combo = new TextFieldComboBox;
     do { // Set up items and current index
         QStringList values;
         QStringList displayTexts;
@@ -211,7 +207,7 @@ QWidget *CustomWizardFieldPage::registerComboBox(const QString &fieldName,
             break;
         combo->setCurrentIndex(currentIndex);
     } while (false);
-    registerField(fieldName, combo, "text", SIGNAL(text4Changed(QString)));
+    registerField(fieldName, combo, "indexText", SIGNAL(text4Changed(QString)));
     // Connect to completeChanged() for derived classes that reimplement isComplete()
     connect(combo, &TextFieldComboBox::text4Changed, this, &QWizardPage::completeChanged);
     return combo;
@@ -220,7 +216,7 @@ QWidget *CustomWizardFieldPage::registerComboBox(const QString &fieldName,
 QWidget *CustomWizardFieldPage::registerTextEdit(const QString &fieldName,
                                                  const CustomWizardField &field)
 {
-    QTextEdit *textEdit = new QTextEdit;
+    auto textEdit = new QTextEdit;
     // Suppress formatting by default (inverting QTextEdit's default value) when
     // pasting from Bug tracker, etc.
     const bool acceptRichText = field.controlAttributes.value(QLatin1String("acceptRichText")) == QLatin1String("true");
@@ -236,7 +232,7 @@ QWidget *CustomWizardFieldPage::registerTextEdit(const QString &fieldName,
 QWidget *CustomWizardFieldPage::registerPathChooser(const QString &fieldName,
                                                  const CustomWizardField &field)
 {
-    PathChooser *pathChooser = new PathChooser;
+    auto pathChooser = new PathChooser;
     const QString expectedKind = field.controlAttributes.value(QLatin1String("expectedkind")).toLower();
     if (expectedKind == QLatin1String("existingdirectory"))
         pathChooser->setExpectedKind(PathChooser::ExistingDirectory);
@@ -264,9 +260,9 @@ QWidget *CustomWizardFieldPage::registerCheckBox(const QString &fieldName,
                                                  const QString &fieldDescription,
                                                  const CustomWizardField &field)
 {
-    typedef CustomWizardField::ControlAttributeMap::const_iterator AttributeMapConstIt;
+    using AttributeMapConstIt = CustomWizardField::ControlAttributeMap::const_iterator;
 
-    TextFieldCheckBox *checkBox = new TextFieldCheckBox(fieldDescription);
+    auto checkBox = new TextFieldCheckBox(fieldDescription);
     const bool defaultValue = field.controlAttributes.value(QLatin1String("defaultvalue")) == QLatin1String("true");
     checkBox->setChecked(defaultValue);
     const AttributeMapConstIt trueTextIt = field.controlAttributes.constFind(QLatin1String("truevalue"));
@@ -275,7 +271,7 @@ QWidget *CustomWizardFieldPage::registerCheckBox(const QString &fieldName,
     const AttributeMapConstIt falseTextIt = field.controlAttributes.constFind(QLatin1String("falsevalue"));
     if (falseTextIt != field.controlAttributes.constEnd()) // Also set empty texts
         checkBox->setFalseText(falseTextIt.value());
-    registerField(fieldName, checkBox, "text", SIGNAL(textChanged(QString)));
+    registerField(fieldName, checkBox, "compareText", SIGNAL(textChanged(QString)));
     // Connect to completeChanged() for derived classes that reimplement isComplete()
     connect(checkBox, &TextFieldCheckBox::textChanged, this, &QWizardPage::completeChanged);
     return checkBox;
@@ -284,13 +280,13 @@ QWidget *CustomWizardFieldPage::registerCheckBox(const QString &fieldName,
 QWidget *CustomWizardFieldPage::registerLineEdit(const QString &fieldName,
                                                  const CustomWizardField &field)
 {
-    QLineEdit *lineEdit = new QLineEdit;
+    auto lineEdit = new QLineEdit;
 
     const QString validationRegExp = field.controlAttributes.value(QLatin1String("validator"));
     if (!validationRegExp.isEmpty()) {
-        QRegExp re(validationRegExp);
+        QRegularExpression re(validationRegExp);
         if (re.isValid())
-            lineEdit->setValidator(new QRegExpValidator(re, lineEdit));
+            lineEdit->setValidator(new QRegularExpressionValidator(re, lineEdit));
         else
             qWarning("Invalid custom wizard field validator regular expression %s.", qPrintable(validationRegExp));
     }
@@ -364,8 +360,8 @@ void CustomWizardFieldPage::cleanupPage()
         PathChooserData &ped = m_pathChoosers[i];
         QString defaultText = ped.defaultText;
         CustomWizardContext::replaceFields(m_context->baseReplacements, &defaultText);
-        if (ped.pathChooser->path() != ped.defaultText)
-            ped.userChange = ped.pathChooser->path();
+        if (ped.pathChooser->filePath().toString() != ped.defaultText)
+            ped.userChange = ped.pathChooser->filePath().toString();
         else
             ped.userChange.clear();
     }
@@ -439,7 +435,7 @@ CustomWizardPage::CustomWizardPage(const QSharedPointer<CustomWizardContext> &ct
 
 QString CustomWizardPage::path() const
 {
-    return m_pathChooser->path();
+    return m_pathChooser->filePath().toString();
 }
 
 void CustomWizardPage::setPath(const QString &path)
